﻿#region Copyright © convivo GmbH
// ------------------------------------------------------------------------------------------------------------------------
// <copyright file="BaseController.cs.cs" company="convivo GmbH">
//    Copyright © 2013 convivo GmbH
// </copyright>
// <creator name="convivo GmbH"/>
// ------------------------------------------------------------------------------------------------------------------------
#endregion

namespace Fruitmonitoring.Controllers.General
{
    #region usings
    using System;
    using System.Collections.Generic;
    using System.Diagnostics;
    using System.Globalization;
    using System.Linq;
    using System.Threading;
    using System.Web;
    using System.Web.Mvc;
    using System.Web.Routing;

    using Fruitmonitoring.Helper.Export;
    using Fruitmonitoring.Helper.Utility;
    using Fruitmonitoring.Helper.Utility.Email;
    using Fruitmonitoring.Infrastructure.ActionAttributes;
    using Fruitmonitoring.Models;

    using log4net;

    #endregion

    /// <summary>
    /// Every controller must inherit from this "base controller"!.
    /// </summary>
    public abstract class BaseController : Controller
    {
        #region variables

        // database object
        /// <summary>
        /// The _db.
        /// </summary>
        private Entities _db;

        /// <summary>
        /// Gets the db.
        /// </summary>
        public Entities Db
        {
            get
            {
                if (_db == null)
                {
                    _db = new Entities();
                }

                return _db;
            }
        }

        // initialize logger
        /// <summary>
        /// The error log.
        /// </summary>
        protected readonly ILog errorLog = LogManager.GetLogger("ErrorLogger");

        /// <summary>
        /// The sample log.
        /// </summary>
        protected readonly ILog sampleLog = LogManager.GetLogger("SampleLogger");

        /// <summary>
        /// The login log.
        /// </summary>
        protected readonly ILog loginLog = LogManager.GetLogger("LoginLogger");

        // sampleLog.Info(new Fruitmonitoring.Utility.Logger.Sample("IM000AD112345", Fruitmonitoring.Utility.Logger.Sample.COMMAND.INSERT));
        // loginLog.Info(new Fruitmonitoring.Utility.Logger.Login("sjohnke","234234", true));

        // initialize error/warning/info lists for frontend frontend
        /// <summary>
        /// The errors.
        /// </summary>
        protected readonly List<string> errors = new List<string>();

        /// <summary>
        /// The warnings.
        /// </summary>
        protected readonly List<string> warnings = new List<string>();

        /// <summary>
        /// The info.
        /// </summary>
        protected readonly List<string> info = new List<string>();

        /// <summary>
        /// The infocontainername.
        /// </summary>
        public static readonly string INFOCONTAINERNAME = "infoContainer"; // replace this value in "fruitmonitoring.js" manually!
        #endregion

        #region overrides
        /// <summary>
        /// Called after the action method is invoked.
        /// </summary>
        /// <param name="filterContext">
        /// Information about the current request and action.
        /// </param>
        protected override void OnActionExecuted(ActionExecutedContext filterContext)
        {
            // error/warning/info list handling start
            if (errors.Count > 0)
            {
                HttpContext.Session.Add("errors", errors);
            }

            if (warnings.Count > 0)
            {
                HttpContext.Session.Add("warnings", warnings);
            }

            if (info.Count > 0)
            {
                HttpContext.Session.Add("info", info);
            }

            // error/warning/info list handling end

            // culture handling start
            // more information: http://afana.me/post/aspnet-mvc-internationalization.aspx
            // check if a different language "view" is available and load it
            // Is it View ?
            var view = filterContext.Result as ViewResultBase;
            if (view == null)
            {
                // if not exit
                return;
            }

            string cultureName = Thread.CurrentThread.CurrentCulture.Name;

            // e.g. "en-US" // filterContext.HttpContext.Request.UserLanguages[0]; // needs validation return "en-us" as default            

            // Is it default culture? exit
            if (cultureName == CultureHelper.GetDefaultCulture())
            {
                return;
            }

            // Are views implemented separately for this culture?  if not exit
            bool viewImplemented = CultureHelper.IsViewSeparate(cultureName);
            if (viewImplemented == false)
            {
                return;
            }

            string viewName = view.ViewName;

            int i = 0;

            if (string.IsNullOrEmpty(viewName))
            {
                viewName = string.Format("{0}.{1}", filterContext.RouteData.Values["action"], cultureName); // Index.en-US
            }
            else if ((i = viewName.IndexOf('.')) > 0)
            {
                // contains . like "Index.cshtml"                
                viewName = viewName.Substring(0, i + 1) + cultureName + viewName.Substring(i);
            }
            else
            {
                viewName += "." + cultureName; // e.g. "Index" ==> "Index.en-Us"
            }

            view.ViewName = viewName;

            filterContext.Controller.ViewBag._culture = "." + cultureName;

            // culture handling end
            base.OnActionExecuted(filterContext);
        }

        /// <summary>
        /// Initializes data that might not be available when the constructor is called.
        /// </summary>
        /// <param name="requestContext">
        /// The HTTP context and route data.
        /// </param>
        protected override void Initialize(RequestContext requestContext)
        {
            // culture cookie handling start
            // more information: http://afana.me/post/aspnet-mvc-internationalization.aspx
            // load culture from cookie or http-header and set this information for this thread
            string cultureName = null;

            try
            {
                // Attempt to read the culture cookie from Request
                HttpCookie cultureCookie = requestContext.HttpContext.Request.Cookies["_culture"];
                if (cultureCookie != null)
                {
                    cultureName = cultureCookie.Value;
                }
                else
                {
                    cultureName = requestContext.HttpContext.Request.UserLanguages[0];

                    // obtain it from HTTP header AcceptLanguages if the user visits the website the first time
                }
            }
            catch (Exception ex)
            {
                Debug.WriteLine("Warning in getting culture. " + ex);
            }

            // Validate culture name
            cultureName = CultureHelper.GetValidCulture(cultureName); // This is safe

            // Modify current thread's culture            
            ChangeCulture(cultureName);

            // save culture in viewbag so we can use it easy in the view
            ViewBag._culture = Thread.CurrentThread.CurrentCulture.Name;

            // culture cookie handling end
            base.Initialize(requestContext);
        }
        #endregion

        #region actionresults
        /// <summary>
        /// Renders the partial view, which contains error messages
        /// use this to provide client errors
        /// you can handle the ajax.error function to use the result and show the errors on the website
        /// e.g. $("#" + Fruitmonitoring.Controllers.General.BaseController.INFOCONTAINERNAME).replaceWith(xhr.responseText).
        /// </summary>
        /// <returns>
        /// Partial html with error messages.
        /// </returns>
        [AjaxOrChildActionOnly]
        public ActionResult ErrorMessagesPartial()
        {
            return null;
        }

        /// <summary>
        /// Renders the partial view, which contains info messages
        /// use this to provide client errors.
        /// </summary>
        /// <returns>
        /// Partial html with info messages.
        /// </returns>
        [AjaxOrChildActionOnly]
        public ActionResult InfoMessagesPartial()
        {
            return null;
        }

        /// <summary>
        /// Renders the partial view, which contains error messages
        /// use this to provide client errors
        /// you can handle the ajax.error function to use the result and show the errors on the website
        /// e.g. $("#errorContainer").replaceWith(xhr.responseText).
        /// </summary>
        /// <param name="error">
        /// Error message which is append to the error list.
        /// </param>
        /// <returns>
        /// Partial html with error messages.
        /// </returns>
        [AjaxOrChildActionOnly]
        public ActionResult ErrorMessagesPartial(string error)
        {
            return null;
        }

        /// <summary>
        /// Initializes the export.
        /// </summary>
        /// <param name="model">
        /// The model.
        /// </param>
        /// <param name="exportType">
        /// Type of the export.
        /// </param>
        /// <param name="columnNames">
        /// Names of all columnNamesArray (separated by "[,]").
        /// </param>
        /// <param name="columnTexts">
        /// Texts of all columnNamesArray (separated by "[,]").
        /// </param>
        /// <param name="filter">
        /// Gridview filter expression.
        /// </param>
        /// <param name="sortColumnName">
        /// Sorted column name.
        /// </param>
        /// <param name="sortOrder">
        /// Sort order "Descending" or "Ascending".
        /// </param>
        /// <param name="applySorting">
        /// Use active sorting?.
        /// </param>
        /// <param name="applyFilter">
        /// Use active filtering?.
        /// </param>
        /// <param name="exportOptions">
        /// The export Options.
        /// </param>
        /// <returns>
        /// Excel file.
        /// </returns>
        [ChildActionOnly]
        protected virtual ActionResult Export<T>(
            IQueryable<T> model,
            string exportType,
            string columnNames,
            string columnTexts,
            string filter = "",
            string sortColumnName = "",
            string sortOrder = "Descending",
            bool applySorting = true,
            bool applyFilter = true,
            ExportOptions exportOptions = null)
        {
            return null;
        }
        #endregion

        #region methods
        /// <summary>
        /// Cleanup all errors, warnings and infos.
        /// </summary>
        public void cleanErrorsWaringsInfos()
        {
        }

        /// <summary>
        /// If validation fails, call this function
        /// use this function only in ajax requests
        /// the error code is handled by the ajax call to identify that an validation error occurs.
        /// </summary>
        public void SetValidateFailedStatus()
        {
        }

        /// <summary>
        /// Send mail.
        /// </summary>
        /// <param name="viewDict">
        /// ViewDataDictionary with the Model and addiotional data.
        /// </param>
        /// <param name="tempDict">
        /// TempDataDictionary.
        /// </param>
        /// <param name="subject">
        /// Email subject.
        /// </param>
        /// <param name="recipients">
        /// String with Comma separated recipients.
        /// </param>
        /// <param name="mailTemplate">
        /// Name of the mailTemplate.
        /// </param>
        /// <param name="recipientsCC">
        /// String with Comma separated recipients CC.
        /// </param>
        /// <param name="recipientsBC">
        /// String with Comma separated recipients BC.
        /// </param>
        /// <returns>
        /// True if mail was send.
        /// </returns>
        public virtual bool sendMail(
            ViewDataDictionary viewDict,
            TempDataDictionary tempDict,
            string subject,
            string recipients,
            string mailTemplate,
            string recipientsCC = null,
            string recipientsBC = null)
        {
            return false;
        }

        /// <summary>
        /// The dispose.
        /// </summary>
        /// <param name="disposing">
        /// The disposing.
        /// </param>
        protected override void Dispose(bool disposing)
        {
            base.Dispose(disposing);
            Db.Dispose();
        }

        /// <summary>
        /// Modify current thread culture.
        /// </summary>
        /// <param name="cultureName">
        /// E.g. en-GB.
        /// </param>
        public void ChangeCulture(string cultureName)
        {
        }
        #endregion
    }
}