﻿#region Copyright © convivo GmbH
// ------------------------------------------------------------------------------------------------------------------------
// <copyright file="FilterExpression.cs.cs" company="convivo GmbH">
//    Copyright © 2013 convivo GmbH
// </copyright>
// <creator name="convivo GmbH"/>
// ------------------------------------------------------------------------------------------------------------------------
#endregion

namespace Fruitmonitoring.Helper.Utility
{
    #region usings
    using System;
    using System.ComponentModel;
    using System.Globalization;
    using System.Linq;
    using Fruitmonitoring.Helper.Exceptions;
    using Languages;
    using Languages.Components;
    using log4net;

    #endregion

    /// <summary>
    /// An object of this class represents an filter which can be used to build the xml for the stored procedure.
    /// </summary>
    public class FilterExpression
    {
        #region variables
        /// <summary>
        /// The date format that devexpress is providing. Use this to transform into own dateformats.
        /// </summary>
        public static readonly string FilterDateFormat = @"\#yyyy-MM-dd\#";

        // column name
        /// <summary>
        /// Gets the key.
        /// </summary>
        public string Key { get; }

        // filter value
        /// <summary>
        /// Gets the value.
        /// </summary>
        public string Value { get; }

        // sql operator type
        /// <summary>
        /// Gets the operator type.
        /// </summary>
        public string OperatorType { get; }

        /// <summary>
        /// ColumnPattern, needed for date filters to accomodate for problem
        /// with ambigous column names, see FM-77.
        /// </summary>
        public string Pattern { get; private set; } = string.Empty;
        #endregion

        #region constructors
        /// <summary>
        /// Initializes a new instance of the <see cref="FilterExpression"/> class.
        /// </summary>
        public FilterExpression()
        {
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="FilterExpression"/> class.
        /// </summary>
        /// <param name="columnName">
        /// The column name.
        /// </param>
        /// <param name="value">
        /// The value.
        /// </param>
        /// <param name="operatorType">
        /// The operator type.
        /// </param>
        /// <param name="modelType">
        /// The model type.
        /// </param>
        public FilterExpression(string columnName, string value, string operatorType, Type modelType)
        {
            // if cast succeed, set values
            Key = columnName;
            if (operatorType.Equals("IN"))
            {
                Value = $"{value}";
            }
            else
            {
                // remove enclosing ' if needed
                if (value.StartsWith("'") && value.EndsWith("'"))
                {
                    value = value.Remove(0, 1);
                    value = value.Remove(value.Length - 1, 1);
                }

                var addApostrophe = true;
                ValidateFilterValue(modelType, ref columnName, ref value, ref addApostrophe, ref operatorType);
                Key = columnName;
                if (addApostrophe)
                {
                    Value = $"'{value}'";
                }
                else
                {
                    Value = $"{value}"; // No quotes since it's numeric or date
                }
            }

            OperatorType = operatorType;
        }

        #endregion

        #region public methods

        /// <summary>
        /// Convert an "filter" value to sql syntax
        /// e.g. for "startswith" do: filtervalue = "test"   ----&gt; sql = "test%".
        /// </summary>
        /// <param name="value">
        /// Filter value.
        /// </param>
        /// <param name="operatorType">
        /// Devexpress gridview operator.
        /// </param>
        /// <returns>
        /// Filter string in sql syntax.
        /// </returns>
        public string ConcatSqlOperatorWithValue(string value, string operatorType)
        {
            switch (operatorType.ToLower())
            {
                case "startswith":
                    return $"{value}%";
                case "endswith":
                    return $"%{value}";
                case "equal":
                    return $"{value}";
                case "notequal":
                    return $"{value}";
                case "contains":
                    return $"%{value}%";
                case "not": // not contains
                    return $"%{value}%";
                case "less":
                case "lessorequal":
                case "greater":
                case "greaterorequal":
                    return $"{value}";
                default:
                    return $"%{value}%";
            }
        }

        /// <summary>
        /// Parse an filter-operator which is given from devexpress-gridview to an sql-operator for the stored procedure.
        /// </summary>
        /// <param name="operatorType">
        /// Devexpress-gridview operator.
        /// </param>
        /// <returns>
        /// Sql operator.
        /// </returns>
        public string OperatorToSql(string operatorType)
        {
            string sql;

            switch (operatorType.ToLower())
            {
                case "startswith":
                    sql = "LIKE";
                    break;
                case "endswith":
                    sql = "LIKE";
                    break;
                case "equal":
                    sql = "=";
                    break;
                case "notequal":
                    sql = "!=";
                    break;
                case "contains":
                    sql = "LIKE";
                    break;
                case "not": // not contains
                    sql = "NOT LIKE";
                    break;
                case "less":
                    sql = "&lt;";
                    break;
                case "lessorequal":
                    sql = "&lt;=";
                    break;
                case "greater":
                    sql = "&gt;";
                    break;
                case "greaterorequal":
                    sql = "&gt;=";
                    break;
                default:
                    sql = "LIKE";
                    break;
            }

            return sql;
        }

        /// <summary>
        /// The to string.
        /// </summary>
        /// <returns>
        /// The <see cref="string"/>.
        /// </returns>
        public override string ToString()
        {
            return $"Key: '{Key}' Value: '{Value}' SqlExpression: '{OperatorType}'";
        }
        #endregion

        #region methods
        /// <summary>
        /// Try to cast the given value to the specified datatype.
        /// </summary>
        /// <param name="modelType">
        /// Modeltype that holds the property.
        /// </param>
        /// <param name="propertyName">
        /// Property name in the model.
        /// </param>
        /// <param name="value">
        /// Property value.
        /// </param>
        /// <param name="addApostrophe">
        /// The add Apostrophe.
        /// </param>
        /// <param name="operatorType">
        /// The filter opertator.
        /// </param>
        private void ValidateFilterValue(Type modelType, ref string propertyName, ref string value, ref bool addApostrophe, ref string operatorType)
        {
            try
            {
                var propertyType = modelType.GetProperty(propertyName).PropertyType;

                // empty and null strings are always valid
                if (!string.IsNullOrEmpty(value))
                {
                    // handle dates
                    if (propertyType == typeof(DateTime) || propertyType == typeof(DateTime?))
                    {
                        // add cast operator for db. db will ignore the timestamp
                        value = $"CAST('{ParseDateToDBDate(value)}' AS DATE)";

                        // The pattern will leads to casting of the column name on the sql server.
                        // CAST is necessary because the SP will cast the underlying value,
                        // the additional conversion leads to dates thats are null temporarily being converted to 01-01-1900, which the filter can work with
                        Pattern = $"CAST({GenerateNullValuePattern("CONVERT(smalldatetime, 0)) as DATE")}";

                        addApostrophe = false;
                    }

                    // handle guid
                    else if (propertyType == typeof(Guid) || propertyType == typeof(Guid?))
                    {
                        // We can't search for "equal/notequal" for invalid guid's in DB.
                        // The DB is going to cast the value and will throw an exception if we provide an invalid guid.
                        // Check for valid guid's here and send an Guid.Empty to DB if user input is not valid.
                        // Invalid user input will result in an emty result list as an empty guid will.
                        if (operatorType.Equals("=") || operatorType.Equals("!="))
                        {
                            Guid id;
                            if (!Guid.TryParse(value, out id))
                            {
                                value = Guid.Empty.ToString();
                            }
                        }
                        
                    }

                    // handle bool
                    else if (propertyType == typeof(bool) || propertyType == typeof(bool?))
                    {
                        if (!value.Equals("1") && !value.Equals("0"))
                        {
                            // ReSharper disable once ReturnValueOfPureMethodIsNotUsed
                            bool.Parse(value);
                        }

                        Pattern = GenerateNullValuePattern("CAST(0 as BIT)");
                    }

                    // handle decimal
                    else if (propertyType == typeof(decimal) || propertyType == typeof(decimal?))
                    {
                        value = ParseDecimalToDBDecimal(value);

                        // ReSharper disable once ReturnValueOfPureMethodIsNotUsed
                        decimal.Parse(value, NumberStyles.Number);
                        addApostrophe = false;
                    }

                    // handle Integer
                    else if (propertyType == typeof(int) || propertyType == typeof(int?))
                    {
                        // ReSharper disable once ReturnValueOfPureMethodIsNotUsed
                        int.Parse(value);
                        addApostrophe = false;

                        Pattern = GenerateNullValuePattern("0"); // TODO: maybe this is not need if DB changes are complete
                    }

                    // handle string
                    else if (propertyType == typeof(string))
                    {
                        Pattern = GenerateNullValuePattern($"'{string.Empty}'");
                    }

                    // try to handle all other datatypes
                    else
                    {
                        // handle nullables
                        if (propertyType.IsGenericType && propertyType.GetGenericTypeDefinition() == typeof(Nullable<>))
                        {
                            var nullableConverter = new NullableConverter(propertyType);
                            var nullableType = nullableConverter.UnderlyingType;

                            // ReSharper disable once ReturnValueOfPureMethodIsNotUsed
                            Convert.ChangeType(value, nullableType);
                        }

                        // handle none-nullables
                        else
                        {
                            // ReSharper disable once ReturnValueOfPureMethodIsNotUsed
                            Convert.ChangeType(value, propertyType);
                        }
                    }
                }
            }
            catch (NullReferenceException ex)
            {
                LogManager.GetLogger("ErrorLogger").Error($"Error: {ex} \nIs GetProperty() null? propertyName = {propertyName}");
            }
            catch (Exception e)
            {
                LogManager.GetLogger("ErrorLogger").Error("Filter value was invalid: " + (value ?? string.Empty) + "Exception: " + e);
                throw new InvalidFilterException(string.Format(Gridview.FilterError, value));
            }
        }

        /// <summary>
        /// Generates an "null" check that can be read by the database.
        /// Moreover a defaultvalue is specified, that is set for null values in the database.
        /// </summary>
        /// <param name="defaultValue">The default value that should be set.</param>
        /// <returns>The string condition with the null check and the default value that can be read by the database.</returns>
        private string GenerateNullValuePattern(string defaultValue)
        {
            return $"ISNULL(%COLUMNNAME%,{defaultValue})";
        }

        /// <summary>
        /// Parse a decimal value to our conventions.
        /// </summary>
        /// <param name="value">
        /// The value.
        /// </param>
        /// <returns>
        /// Decimal without "m" at the end.
        /// </returns>
        private string ParseDecimalToDBDecimal(string value)
        {
            if (char.IsLetter(value.ElementAt(value.Length - 1)))
            {
                value = value.Remove(value.Length - 1, 1);
            }

            return value;
        }

        /// <summary>
        /// Parses a "gridview date" to "db date"
        /// incomming format "#yyyy-mm-dd#" or "27.08.2015"
        /// outgoing format "yyyyMMdd".
        /// </summary>
        /// <param name="value">
        /// The value.
        /// </param>
        /// <returns>
        /// Db date format.
        /// </returns>
        private string ParseDateToDBDate(string value)
        {
            var datePatternGridView = FilterDateFormat; // "MM/dd/yyyy HH:mm:ss";
            var datePatternDateTime = Global.DateFormat;

            DateTime parsedDate;

            // parse dates that comes from the gridview
            if (DateTime.TryParseExact(
                value,
                datePatternGridView,
                null,
                DateTimeStyles.None,
                out parsedDate))
            {
                return string.Format(Global.DateFormatStringDBCSharp, parsedDate);
            }

            // parse dates that comes from the dateTime component
            if (DateTime.TryParseExact(
                value,
                datePatternDateTime,
                null,
                DateTimeStyles.None,
                out parsedDate))
            {
                return string.Format(Global.DateFormatStringDBCSharp, parsedDate);
            }

            throw new Exception("Date can't parsed: " + value);
        }
        #endregion
    }
}