﻿#region Copyright © convivo GmbH
// ------------------------------------------------------------------------------------------------------------------------
// <copyright file="SampleOverdueMailer.cs.cs" company="convivo GmbH">
//    Copyright © 2014 convivo GmbH
// </copyright>
// <creator name="convivo GmbH"/>
// ------------------------------------------------------------------------------------------------------------------------
#endregion

namespace Fruitmonitoring.Helper.Utility.Tasks.Tasks.SampleOverdueTask
{
    #region usings
    using System;
    using System.Collections.Generic;
    using System.Configuration;
    using System.Linq;
    using System.Threading;
    using System.Web.Configuration;
    using System.Web.Mvc;

    using DevExpress.Data.Filtering;

    using Fruitmonitoring.Helper.Utility.Email;
    using Fruitmonitoring.Helper.Utility.Tasks.SampleOverdueTask;
    using Fruitmonitoring.Models;

    using log4net;

    using Languages.Emails.Samples;

    using Sample = Languages.ModelData.Sample.Sample;

    #endregion

    /// <summary>
    /// Class for filtering/controlling notification emails sent for
    /// overdue laboratory results. Scheudeling is done in FluentRegistry.cs.
    /// </summary>
    public class SampleOverdueMailer : IDisposable
    {
        #region Delegates

        // delegate for method to return retailer mailing list
        /// <summary>
        /// The del function.
        /// </summary>
        /// <param name="retailer">
        /// The retailer.
        /// </param>
        public delegate List<USP_Sample_Sel_Result> DelFunction(Guid retailer);
        #endregion

        #region EXAMINATIONTYPE enum

        #region Enums
        /// <summary>
        /// The examinationtype.
        /// </summary>
        public enum EXAMINATIONTYPE
        {
            /// <summary>
            /// The defaultexamination.
            /// </summary>
            DEFAULTEXAMINATION,

            /// <summary>
            /// The additionalexamination.
            /// </summary>
            ADDITIONALEXAMINATION,

            /// <summary>
            /// The singleexamination.
            /// </summary>
            SINGLEEXAMINATION,
            /// <summary>
            /// The no examination 
            /// </summary>
            NOEXAMINATIONTYPE
        }
        #endregion

        #endregion

        #region Properties

        #region Constants
        /// <summary>
        /// The email template.
        /// </summary>
        private const string emailTemplate = "~/Views/Shared/Fruitmonitoring/Emails/EmailSampleOverdue.cshtml";
        #endregion

        #endregion

        #region Constructors and Destructors
        /// <summary>
        /// Initializes a new instance of the <see cref="SampleOverdueMailer"/> class.
        /// notifier is class for summary of emails
        /// sent (successfull/failure count) and simple
        /// info mail is sent to Admin.
        /// </summary>
        public SampleOverdueMailer()
        {
            db = new Entities();
            errorLog = LogManager.GetLogger("LabSampleEmailLogger");
            notifier = new AdminEmailer();
            notifier.TaskName = Thread.CurrentThread.Name;
        }
        #endregion

        #region Fields
        /// <summary>
        /// The error log.
        /// </summary>
        protected readonly ILog errorLog = LogManager.GetLogger("LabSampleEmailLogger");

        /// <summary>
        /// The notifier.
        /// </summary>
        private readonly AdminEmailer notifier;

        /// <summary>
        /// The send status.
        /// </summary>
        private readonly string sendStatus = WebConfigurationManager.AppSettings["OverdueSampleTaskEnabled"];

        /// <summary>
        /// The db.
        /// </summary>
        private Entities db;
        #endregion

        #region Public Methods and Operators
        /// <summary>
        /// Db checked for overdue (&gt; 21 calendar days) additional sample examination, email
        /// notifications sent out to corresponding Admin/Reatailer/Laboratory, unless default
        /// examination is also true in which case the sample will already be included in default
        /// mail list.
        /// </summary>
        public void ckForAdditionalExaminationSamples()
        {
            if (isTaskActivated() && isValidDay())
            {
                var adminAndLabList = getAdditionalExaminationList();
                DelFunction delRetailerList = getAdditionalExaminationListForRetailer;

                if (adminAndLabList.Count > 0)
                {
                    sendAllMails(adminAndLabList, delRetailerList, EXAMINATIONTYPE.ADDITIONALEXAMINATION);
                }
            }
        }

        /// <summary>
        /// Db checked for overdue (&gt; 8 calendar days) default sample examination, email
        /// notifications sent out to corresponding Admin/Reatailer/Laboratory.
        /// </summary>
        public void ckForDefaultExaminationSamples()
        {
            if (isTaskActivated() && isValidDay())
            {
                var adminAndLabList = getDefaultExaminationList();
                DelFunction delRetailerList = getDefaultExaminationListForRetailer;

                if (adminAndLabList.Count > 0)
                {
                    sendAllMails(adminAndLabList, delRetailerList, EXAMINATIONTYPE.DEFAULTEXAMINATION);
                }
            }
        }

        /// <summary>
        /// Db checked for overdue (&gt; 21 calendar days) single sample examination, email
        /// notifications sent out to corresponding Admin/Reatailer/Laboratory.
        /// </summary>
        public void ckForSingleExaminationSamples()
        {
            if (isTaskActivated() && isValidDay())
            {
                var adminAndLabList = getSingleExaminationList();
                DelFunction delRetailerList = getSingleExaminationListForRetailer;

                if (adminAndLabList.Count > 0)
                {
                    sendAllMails(adminAndLabList, delRetailerList, EXAMINATIONTYPE.SINGLEEXAMINATION);
                }
            }
        }


        /// <summary>
        /// Db checked for overdue (&gt; 8 calendar days) single sample examination, email
        /// notifications sent out to corresponding Admin/Retailer/Laboratory.
        /// </summary>
        public void ckForNoExaminationTypeSamples()
        {
            if (isTaskActivated() && isValidDay())
            {
                var adminAndLabList = getNoExaminationTypeList();
                DelFunction delRetailerList = getNoExaminationTypeListForRetailer;

                if (adminAndLabList.Count > 0)
                {
                    sendAllMails(adminAndLabList, delRetailerList, EXAMINATIONTYPE.NOEXAMINATIONTYPE);
                }
            }
        }

        /// <summary>
        /// The dispose.
        /// </summary>
        public void Dispose()
        {
            if (db != null)
            {
                db.Dispose();
                db = null;
            }
        }
        #endregion

        #region Methods
        /// <summary>
        /// Get a language value for the given examination type.
        /// </summary>
        /// <param name="examinationType">
        /// The examination type.
        /// </param>
        /// <returns>
        /// A user readable value for the examination type.
        /// </returns>
        private string getExaminationTypeLanguageValue(EXAMINATIONTYPE examinationType)
        {
            switch (examinationType)
            {
                case EXAMINATIONTYPE.DEFAULTEXAMINATION:
                    return Sample.DefaultExamination;
                case EXAMINATIONTYPE.ADDITIONALEXAMINATION:
                    return Sample.AdditionalExamination;
                case EXAMINATIONTYPE.SINGLEEXAMINATION:
                    return Sample.SingleExamination;
                case EXAMINATIONTYPE.NOEXAMINATIONTYPE:
                    return Sample.NoExamination;
                default:
                    return string.Empty;
            }
        }

        /// <summary>
        /// Get task activation
        /// the task can activated/deactivated in the web.conf.
        /// </summary>
        /// <returns>
        /// Ture if task is activated, otherwise false.
        /// </returns>
        private bool isTaskActivated()
        {
            try
            {
                if (bool.Parse(sendStatus))
                {
                    return true;
                }
                else
                {
                    return false;
                }
            }
            catch (ArgumentNullException)
            {
                return false;
            }
            catch (FormatException)
            {
                return false;
            }
        }

        /// <summary>
        /// Should task be executed on this day.
        /// </summary>
        /// <returns>
        /// Returns true if task should executed, otherwise false.
        /// </returns>
        private bool isValidDay()
        {
            if (DateTime.Now.DayOfWeek == DayOfWeek.Sunday)
            {
                return false;
            }
            else
            {
                return true;
            }
        }

        /// <summary>
        /// All mailing methods for overdue samples called.
        /// </summary>
        /// <param name="adminAndLabList">
        /// List of samples that should send to admins, retailers and laboratories.
        /// </param>
        /// <param name="delRetailerList">
        /// </param>
        /// <param name="examinationType">
        /// The type of examination.
        /// </param>
        private void sendAllMails(List<USP_Sample_Sel_Result> adminAndLabList, DelFunction delRetailerList, EXAMINATIONTYPE examinationType)
        {
            sendAdminMail(adminAndLabList, examinationType);
            sendRetailerMail(delRetailerList, examinationType);
            sendLaboratoryMail(adminAndLabList, examinationType);
            using (new CultureHelperLanguageChanger(CultureHelper.GetValidCulture(CultureHelper.Languages.ENGLISH)))
            {
                notifier.sendNotificationMail(db, getExaminationTypeLanguageValue(examinationType), errorLog);
            }
        }

        /// <summary>
        /// Email sent to Fruitmonitoring admin listing all
        /// overdue samples in list.
        /// </summary>
        /// <param name="sampleList">
        /// List of either default, single, or additional examinations.
        /// </param>
        /// <param name="examinationType">
        /// The type of examination.
        /// </param>
        private void sendAdminMail(List<USP_Sample_Sel_Result> sampleList, EXAMINATIONTYPE examinationType)
        {
            // Fruitmonitoring admin receives the mail in german           
            using (new CultureHelperLanguageChanger(CultureHelper.GetValidCulture(CultureHelper.Languages.GERMAN)))
            {
                var emailer = new Emailer();
                var viewDict = new ViewDataDictionary();
                viewDict.Model = sampleList;
                var eMailOptions =
                    new EMailOptions(
                        string.Format(Samples.SampleOverdueSubject, sampleList.Count, getExaminationTypeLanguageValue(examinationType)),
                        ConfigurationManager.AppSettings["AdminMailbox"],
                        emailTemplate,
                        true,
                        viewDict);

                var sendSucceed = emailer.SendMail(eMailOptions, errorLog);
                if (sendSucceed)
                {
                    notifier.AdminMail++;
                }
            }
        }

        /// <summary>
        /// Email sent to laboratory listing the
        /// corresponding overdue samples. Traversing
        /// of list locally to avoid excessive db calls.
        /// </summary>
        /// <param name="sampleList">
        /// List of either default, single, or additional examinations.
        /// </param>
        /// <param name="examinationType">
        /// The type of examination.
        /// </param>
        private void sendLaboratoryMail(List<USP_Sample_Sel_Result> sampleList, EXAMINATIONTYPE examinationType)
        {
            var labMailingList = new HashSet<Guid?>();

            // create lab mailing list
            foreach (var sample in sampleList)
            {
                labMailingList.Add(sample.CompanyLab_id);
            }

            notifier.LabMailTotal = labMailingList.Count;

            // create list of samples for each lab
            foreach (var lab in labMailingList)
            {
                var labSpecificList = sampleList.Where(sample => sample.CompanyLab_id == lab);

                // send email to laboratory in company language
                using (new CultureHelperLanguageChanger(CultureHelper.GetValidCulture(StoredProcedureHelper.GetCompanyISO2(db, (Guid)lab))))
                {
                    var emailer = new Emailer();
                    var viewDict = new ViewDataDictionary();
                    viewDict.Model = labSpecificList;
                    var eMailOptions =
                        new EMailOptions(
                            string.Format(Samples.SampleOverdueSubject, labSpecificList.ToList().Count, getExaminationTypeLanguageValue(examinationType)),
                            StoredProcedureHelper.GetCompaniesEMail(db, (Guid)lab),
                            emailTemplate,
                            true,
                            viewDict);

                    var sendSucceed = emailer.SendMail(eMailOptions, errorLog);
                    if (sendSucceed)
                    {
                        notifier.LabMailSuccess++;
                    }
                }
            }
        }

        /// <summary>
        /// Email sent to each retailer listing the
        /// corresponding overdue samples.
        /// </summary>
        /// <param name="getRetailerList">
        /// The get Retailer List.
        /// </param>
        /// <param name="examinationType">
        /// The type of examination.
        /// </param>
        private void sendRetailerMail(DelFunction getRetailerList, EXAMINATIONTYPE examinationType)
        {
            var retailerList =
                db.USP_Company_Sel(
                    null,
                    null,
                    DbGuidHelper.GetCompanyStatusId(DbGuidHelper.CompanyStatus.ENABLED),
                    DbGuidHelper.GetCompanyTypeId(DbGuidHelper.CompanyTypes.RETAILERCOMPANY).ToString(),
                    null,
                    null,
                    0,
                    null,
                    null,
                    "ENG").ToList<USP_Company_Sel_Result>();

            foreach (var retailer in retailerList)
            {
                var retailerSampleList = getRetailerList(retailer.Company_id);

                if (retailerSampleList.Count != 0)
                {
                    notifier.RetailerMailTotal++;

                    // send email to laboratory in company language
                    using (new CultureHelperLanguageChanger(CultureHelper.GetValidCulture(StoredProcedureHelper.GetCompanyISO2(db, retailer.Company_id))))
                    {
                        var emailer = new Emailer();
                        var viewDict = new ViewDataDictionary();
                        viewDict.Model = retailerSampleList;
                        var eMailOptions =
                            new EMailOptions(
                                string.Format(Samples.SampleOverdueSubject, retailerSampleList.Count, getExaminationTypeLanguageValue(examinationType)),
                                StoredProcedureHelper.GetCompaniesEMail(db, retailer.Company_id),
                                emailTemplate,
                                true,
                                viewDict);

                        var sendSucceed = emailer.SendMail(eMailOptions, errorLog);
                        if (sendSucceed)
                        {
                            notifier.RetailerMailSuccess++;
                        }
                    }
                }
            }
        }
        #endregion

        #region get uspLists
        /// <summary>
        /// Returns all samples where default exam = true and is overdue.
        /// </summary>
        /// <returns>
        /// The <see cref="List"/>.
        /// </returns>
        private List<USP_Sample_Sel_Result> getDefaultExaminationList()
        {
            var xmlFilter = getDefaultxmlFilter();

            return getOverdueSamplesList(xmlFilter, EXAMINATIONTYPE.DEFAULTEXAMINATION);
        }

        /// <summary>
        /// Returns all samples where single exam = true and is overdue.
        /// </summary>
        /// <returns>
        /// The <see cref="List"/>.
        /// </returns>
        private List<USP_Sample_Sel_Result> getSingleExaminationList()
        {
            var xmlFilter = getSinglexmlFilter();

            return getOverdueSamplesList(xmlFilter, EXAMINATIONTYPE.SINGLEEXAMINATION);
        }

        /// <summary>
        /// Returns all samples where additional exam = true,
        /// default exam = false, and is overdue.
        /// </summary>
        /// <returns>
        /// The <see cref="List"/>.
        /// </returns>
        private List<USP_Sample_Sel_Result> getAdditionalExaminationList()
        {
            var xmlFilter = getAdditionalxmlFilter();

            return getOverdueSamplesList(xmlFilter, EXAMINATIONTYPE.ADDITIONALEXAMINATION);
        }

        private List<USP_Sample_Sel_Result> getNoExaminationTypeList()
        {
            var xmlFilter = getNoExaminationTypexmlFilter();

            return getOverdueSamplesList(xmlFilter, EXAMINATIONTYPE.NOEXAMINATIONTYPE);
        }
        /// <summary>
        /// Returns all overdue samples according to the xml filter
        /// </summary>
        /// <param name="xmlFilter">
        /// </param>
        /// <returns>
        /// </returns>
        private List<USP_Sample_Sel_Result> getOverdueSamplesList(string xmlFilter, EXAMINATIONTYPE examinationType)
        {
            var overdueSamplesList =
                db.USP_Sample_Sel(DbGuidHelper.GetAdminAccountId(), null, null, null, null, 0, null, xmlFilter, null).ToList<USP_Sample_Sel_Result>();

            removeFalsePositives(overdueSamplesList, examinationType);

            return overdueSamplesList;
        }

        /// <summary>
        /// Removes entries from the list of overdue samples whose LabEntryDate is '0001-01-01 00:00:00.0000000'.
        /// This is true and valid for samples that are derived from investigation orders,
        /// for these the lab can set the date later.
        /// 
        /// If LabEntryDate is NULL, we will use the CreationDate as base for the overdue calculation.
        /// 
        /// Update 25.07.2017: MinDates arn't saved anymore to DB. We use this function as a backup and for old sample data.
        /// </summary>
        /// <param name="examinationsList">
        /// List of overdue samples
        /// </param>
        private void removeFalsePositives(List<USP_Sample_Sel_Result> examinationsList, EXAMINATIONTYPE examinationType)
        {
            examinationsList.RemoveAll(x => x.LabEntryDate == DateTime.MinValue);
            
            switch (examinationType)
            {
                case EXAMINATIONTYPE.DEFAULTEXAMINATION:
                    examinationsList.RemoveAll(x => x.LabEntryDate == null && !(x.CreationDate <= DateTime.Now.AddDays(-8)));
                    break;
                case EXAMINATIONTYPE.ADDITIONALEXAMINATION:
                    examinationsList.RemoveAll(x => x.LabEntryDate == null && !(x.CreationDate <= DateTime.Now.AddDays(-21)));
                    break;
                case EXAMINATIONTYPE.SINGLEEXAMINATION:
                    examinationsList.RemoveAll(x => x.LabEntryDate == null && !(x.CreationDate <= DateTime.Now.AddDays(-8)));
                    break;
                case EXAMINATIONTYPE.NOEXAMINATIONTYPE:
                    examinationsList.RemoveAll(x => x.LabEntryDate == null && !(x.CreationDate <= DateTime.Now.AddDays(-8)));
                    break;
            }
        }

        /// <summary>
        /// Returns all samples where single exam = true,
        /// comp_id = retailer_id, and is overdue.
        /// </summary>
        /// <param name="retailer">
        /// The retailer.
        /// </param>
        /// <returns>
        /// The <see cref="List"/>.
        /// </returns>
        private List<USP_Sample_Sel_Result> getSingleExaminationListForRetailer(Guid retailer)
        {
            var xmlFilter = getSinglexmlFilter();

            var singleExaminationSampleList = db.USP_Sample_Sel(DbGuidHelper.GetAdminAccountId(), null, retailer, null, null, 0, null, xmlFilter, null).ToList();
            removeFalsePositives(singleExaminationSampleList, EXAMINATIONTYPE.SINGLEEXAMINATION);

            return singleExaminationSampleList;
        }

        /// <summary>
        /// Returns all samples where default exam = true,
        /// comp_id = retailer_id, and is overdue.
        /// </summary>
        /// <param name="retailer">
        /// The retailer.
        /// </param>
        /// <returns>
        /// The <see cref="List"/>.
        /// </returns>
        private List<USP_Sample_Sel_Result> getDefaultExaminationListForRetailer(Guid retailer)
        {
            var xmlFilter = getDefaultxmlFilter();

            var defaultExaminationSampleList = db.USP_Sample_Sel(DbGuidHelper.GetAdminAccountId(), null, retailer, null, null, 0, null, xmlFilter, null).ToList();
            removeFalsePositives(defaultExaminationSampleList, EXAMINATIONTYPE.DEFAULTEXAMINATION);

            return defaultExaminationSampleList;
        }

        /// <summary>
        /// Returns all samples where default exam = false,
        /// additional exam = true, comp_id = retailer_id
        /// and is overdue.
        /// </summary>
        /// <param name="retailer">
        /// The retailer.
        /// </param>
        /// <returns>
        /// The <see cref="List"/>.
        /// </returns>
        private List<USP_Sample_Sel_Result> getAdditionalExaminationListForRetailer(Guid retailer)
        {
            var xmlFilter = getAdditionalxmlFilter();

            var additionalExaminationSampleList = db.USP_Sample_Sel(DbGuidHelper.GetAdminAccountId(), null, retailer, null, null, 0, null, xmlFilter, null).ToList();
            removeFalsePositives(additionalExaminationSampleList, EXAMINATIONTYPE.ADDITIONALEXAMINATION);

            // retrieve samples from db
            return additionalExaminationSampleList;
        }

        private List<USP_Sample_Sel_Result> getNoExaminationTypeListForRetailer(Guid retailer)
        {
            var xmlFilter = getNoExaminationTypexmlFilter();

            var NoExaminationTypeSampleList = db.USP_Sample_Sel(DbGuidHelper.GetAdminAccountId(), null, retailer, null, null, 0, null, xmlFilter, null).ToList();
            removeFalsePositives(NoExaminationTypeSampleList, EXAMINATIONTYPE.NOEXAMINATIONTYPE);

            // retrieve samples from db
            return NoExaminationTypeSampleList;
        }

        #endregion

        #region get xmlFilters
        /// <summary>
        /// The get defaultxml filter.
        /// </summary>
        /// <returns>
        /// The <see cref="string"/>.
        /// </returns>
        private string getDefaultxmlFilter()
        {
            var gridViewFilterHelper = new GridViewFilterHelper();
            var statusFilterExpression = getSampleStatusFilterExpression();

            // generate defaultExamination-filterexpression for xml filter
            var defaultExaminationExpression =
                new BinaryOperator(CustomMetadataProvider.GetMemberName(() => EFHelper.Usp_Sample.DefaultExamination), true, BinaryOperatorType.Equal).ToString(
                    
                    );

            // generate additionalExamination-filterexpression for xml filter
            var additionalExaminationExpression =
                new BinaryOperator(CustomMetadataProvider.GetMemberName(() => EFHelper.Usp_Sample.AdditionalExamination), false, BinaryOperatorType.Equal)
                    .ToString();

            // generate overdueLabEntry-filterexpression for xml filter
            // returns also samples with labEntryDate = NULL
            var overdueLabentryExpression =
                new BinaryOperator(
                    CustomMetadataProvider.GetMemberName(() => EFHelper.Usp_Sample.LabEntryDate),
                    DateTime.Now.AddDays(-8).ToString(FilterExpression.FilterDateFormat),
                    BinaryOperatorType.LessOrEqual).ToString();

            // generate xml filter for sp
            var xmlFilter =
                gridViewFilterHelper.GenerateXmlFilter(
                    string.Format(
                        "{0} AND {1} AND {2} AND {3}",
                        statusFilterExpression,
                        defaultExaminationExpression,
                        additionalExaminationExpression,
                        overdueLabentryExpression),
                    EFHelper.Usp_Sample.GetType());

            return xmlFilter;
        }

        /// <summary>
        /// The get singlexml filter.
        /// </summary>
        /// <returns>
        /// The <see cref="string"/>.
        /// </returns>
        private string getSinglexmlFilter()
        {
            var gridViewFilterHelper = new GridViewFilterHelper();
            var statusFilterExpression = getSampleStatusFilterExpression();

            // generate singleExamination-filterexpression for xml filter
            var singleExaminationExpression =
                new BinaryOperator(CustomMetadataProvider.GetMemberName(() => EFHelper.Usp_Sample.SingleExamination), true, BinaryOperatorType.Equal).ToString();

            // generate additionalExamination-filterexpression for xml filter
            var additionalExaminationExpression =
                new BinaryOperator(CustomMetadataProvider.GetMemberName(() => EFHelper.Usp_Sample.AdditionalExamination), false, BinaryOperatorType.Equal)
                    .ToString();

            // generate overdueLabEntry-filterexpression for xml filter
            // returns also samples with labEntryDate = NULL
            var overdueLabentryExpression =
                new BinaryOperator(
                    CustomMetadataProvider.GetMemberName(() => EFHelper.Usp_Sample.LabEntryDate),
                    DateTime.Now.AddDays(-8).ToString(FilterExpression.FilterDateFormat),
                    BinaryOperatorType.LessOrEqual).ToString();

            // generate xml filter for sp
            var xmlFilter =
                gridViewFilterHelper.GenerateXmlFilter(
                    string.Format(
                        "{0} AND {1} AND {2} AND {3}",
                        statusFilterExpression,
                        singleExaminationExpression,
                        additionalExaminationExpression,
                        overdueLabentryExpression),
                    EFHelper.Usp_Sample.GetType());

            return xmlFilter;
        }

        /// <summary>
        /// The get additionalxml filter.
        /// </summary>
        /// <returns>
        /// The <see cref="string"/>.
        /// </returns>
        private string getAdditionalxmlFilter()
        {
            var gridViewFilterHelper = new GridViewFilterHelper();
            var statusFilterExpression = getSampleStatusFilterExpression();

            // generate additionalExamination-filterexpression for xml filter
            var additionalExaminationExpression =
                new BinaryOperator(CustomMetadataProvider.GetMemberName(() => EFHelper.Usp_Sample.AdditionalExamination), true, BinaryOperatorType.Equal)
                    .ToString();

            // generate overdueLabEntry-filterexpression for xml filter
            // returns also samples with labEntryDate = NULL
            var overdueLabentryExpression =
                new BinaryOperator(
                    CustomMetadataProvider.GetMemberName(() => EFHelper.Usp_Sample.LabEntryDate),
                    DateTime.Now.AddDays(-21).ToString(FilterExpression.FilterDateFormat),
                    BinaryOperatorType.LessOrEqual).ToString();

            // generate xml filter for sp
            var xmlFilter =
                gridViewFilterHelper.GenerateXmlFilter(
                    string.Format("{0} AND {1} AND {2}", statusFilterExpression, additionalExaminationExpression, overdueLabentryExpression),
                    EFHelper.Usp_Sample.GetType());

            return xmlFilter;
        }

        /// <summary>
        /// The get samples without examination type filter.
        /// </summary>
        /// <returns>
        /// The <see cref="string"/>.
        /// </returns>
        private string getNoExaminationTypexmlFilter()
        {
            var gridViewFilterHelper = new GridViewFilterHelper();
            var statusFilterExpression = getSampleStatusFilterExpression();

            // generate additionalExamination-filterexpression for xml filter
            var additionalExaminationExpression =
                new BinaryOperator(CustomMetadataProvider.GetMemberName(() => EFHelper.Usp_Sample.AdditionalExamination), false, BinaryOperatorType.Equal)
                    .ToString();
            var defaultExaminationExpression =
                new BinaryOperator(CustomMetadataProvider.GetMemberName(() => EFHelper.Usp_Sample.DefaultExamination), false, BinaryOperatorType.Equal)
                    .ToString();
            var singleExaminationExpression =
                new BinaryOperator(CustomMetadataProvider.GetMemberName(() => EFHelper.Usp_Sample.SingleExamination), false, BinaryOperatorType.Equal)
                    .ToString();

            // generate overdueLabEntry-filterexpression for xml filter
            // returns also samples with labEntryDate = NULL
            var overdueLabentryExpression =
                new BinaryOperator(
                    CustomMetadataProvider.GetMemberName(() => EFHelper.Usp_Sample.LabEntryDate),
                    DateTime.Now.AddDays(-8).ToString(FilterExpression.FilterDateFormat),
                    BinaryOperatorType.LessOrEqual).ToString();

            // generate xml filter for sp
            var xmlFilter =
                gridViewFilterHelper.GenerateXmlFilter(
                    string.Format(
                        "{0} AND {1} AND {2} AND {3} AND {4}", 
                        statusFilterExpression, 
                        additionalExaminationExpression,
                        defaultExaminationExpression,
                        singleExaminationExpression,
                        overdueLabentryExpression),
                    EFHelper.Usp_Sample.GetType());

            return xmlFilter;
        }

        /// <summary>
        /// The get sample status filter expression.
        /// </summary>
        /// <returns>
        /// The <see cref="string"/>.
        /// </returns>
        private string getSampleStatusFilterExpression()
        {
            // generate samplestatus-filterexpression for xml filter
            var statusFilterExpression =
                new InOperator(
                    CustomMetadataProvider.GetMemberName(() => EFHelper.Usp_Sample.SampleStatus_id),
                    new[]
                    {
                        string.Format("{0}", DbGuidHelper.GetSampleStatusId(DbGuidHelper.SampleStatus.INPROGRESS)),
                        string.Format("{0}", DbGuidHelper.GetSampleStatusId(DbGuidHelper.SampleStatus.SAMPLERECEIPT))
                    }).ToString();
            return statusFilterExpression;
        }
        #endregion
    }
}