﻿#region Copyright © convivo GmbH
// ------------------------------------------------------------------------------------------------------------------------
// <copyright file="EANValidationAttribute.cs.cs" company="convivo GmbH">
//    Copyright © 2014 convivo GmbH
// </copyright>
// <creator name="convivo GmbH"/>
// ------------------------------------------------------------------------------------------------------------------------
#endregion

namespace Fruitmonitoring.Helper.Validation
{
    #region usings
    using System.ComponentModel.DataAnnotations;
    using System.Globalization;

    #endregion

    /// <summary>
    /// Checks ean number.
    /// </summary>
    public class EANValidationAttribute : ValidationAttribute
    {
        #region Public Methods and Operators
        /// <summary>
        /// Custom error message format.
        /// </summary>
        /// <param name="name">
        /// The name.
        /// </param>
        /// <returns>
        /// Formatted error message.
        /// </returns>
        public override string FormatErrorMessage(string name)
        {
            return null;
        }
        #endregion

        #region Methods
        /// <summary>
        /// Validate that ean is correct.
        /// </summary>
        /// <param name="value">
        /// Current validated value.
        /// </param>
        /// <param name="validationContext">
        /// Validation context.
        /// </param>
        /// <returns>
        /// Null if validation succeed, otherwise an Validation Result.
        /// </returns>
        protected override ValidationResult IsValid(object value, ValidationContext validationContext)
        {
            try
            {
                if (value == null)
                {
                    return null;
                }

                var ean = value.ToString();

                if (!string.IsNullOrEmpty(ean))
                {
                    var factor = 0;

                    // do not validate numbers with 14 digits
                    if (ean.Length == 14)
                    {
                        factor = 1;
                    }

                    // calculation is different in 8 or 13 digests
                    // so we need a factor
                    else if (ean.Length == 13)
                    {
                        factor = 0;
                    }
                    else if (ean.Length == 8)
                    {
                        factor = 1;
                    }
                    else
                    {
                        // ean length is not valid
                        return new ValidationResult(FormatErrorMessage(validationContext.DisplayName));
                    }

                    // calculate checksum
                    var checkSum = 0;

                    var eanBoxParts = ean.ToCharArray();

                    // sum up all digits and multiply each value with 3 or 1
                    for (int i = 0; i < eanBoxParts.Length; i++)
                    {
                        var intNumber = (int)char.GetNumericValue(eanBoxParts[i]);

                        if ((i + factor) % 2 == 0)
                        {
                            checkSum += intNumber * 1;
                        }
                        else
                        {
                            checkSum += intNumber * 3;
                        }
                    }

                    // calculate checknumber
                    // get last digit of the checksum
                    var checkNumber = 10 - checkSum % 10;
                    if (checkNumber == 10)
                    {
                        // is correct
                        return null;
                    }
                    else
                    {
                        // is false
                        return new ValidationResult(FormatErrorMessage(validationContext.DisplayName));
                    }
                }
                else
                {
                    return null;
                }
            }
            catch
            {
                return new ValidationResult(FormatErrorMessage(validationContext.DisplayName));
            }
        }
        #endregion
    }
}